﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 비트맵 1
        /// </summary>
        private Bitmap sourceBitmap1 = null;

        /// <summary>
        /// 소스 비트맵 2
        /// </summary>
        private Bitmap sourceBitmap2 = null;

        /// <summary>
        /// 타겟 비트맵
        /// </summary>
        private Bitmap targetBitmap = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.blueBlendTypeComboBox.Items.Add(BitwiseBlendType.NONE);
            this.blueBlendTypeComboBox.Items.Add(BitwiseBlendType.AND );
            this.blueBlendTypeComboBox.Items.Add(BitwiseBlendType.OR  );
            this.blueBlendTypeComboBox.Items.Add(BitwiseBlendType.XOR );

            this.blueBlendTypeComboBox.SelectedIndex = 0;

            this.greenBlendTypeComboBox.Items.Add(BitwiseBlendType.NONE);
            this.greenBlendTypeComboBox.Items.Add(BitwiseBlendType.AND );
            this.greenBlendTypeComboBox.Items.Add(BitwiseBlendType.OR  );
            this.greenBlendTypeComboBox.Items.Add(BitwiseBlendType.XOR );

            this.greenBlendTypeComboBox.SelectedIndex = 0;

            this.redBlendTypeComboBox.Items.Add(BitwiseBlendType.NONE);
            this.redBlendTypeComboBox.Items.Add(BitwiseBlendType.AND );
            this.redBlendTypeComboBox.Items.Add(BitwiseBlendType.OR  );
            this.redBlendTypeComboBox.Items.Add(BitwiseBlendType.XOR );

            this.redBlendTypeComboBox.SelectedIndex = 0;

            this.loadSourceImage1Button.Click                += loadSourceImageButton_Click;
            this.loadSourceImage2Button.Click                += loadSourceImageButton_Click;
            this.blueBlendTypeComboBox.SelectedIndexChanged  += blendTypeComboBox_SelectedIndexChanged;
            this.greenBlendTypeComboBox.SelectedIndexChanged += blendTypeComboBox_SelectedIndexChanged;
            this.redBlendTypeComboBox.SelectedIndexChanged   += blendTypeComboBox_SelectedIndexChanged;
            this.saveImageButton.Click                       += saveImageButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 소스 이미지 로드 버튼 클릭시 처리하기 - loadSourceImageButton_Click(sender, e)

        /// <summary>
        /// 소스 이미지 로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loadSourceImageButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            openFileDialog.Title  = "이미지 파일 로드";
            openFileDialog.Filter = "PNG 파일(*.png)|*.png|JPEG 파일(*.jpg)|*.jpg|비트맵 파일(*.bmp)|*.bmp";

            if(openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if(sender == this.loadSourceImage1Button)
                {
                    StreamReader streamReader = new StreamReader(openFileDialog.FileName);

                    this.sourceBitmap1 = Bitmap.FromStream(streamReader.BaseStream) as Bitmap;

                    streamReader.Close();

                    this.sourceImage1PictureBox.Image = BitmapHelper.CopyBitmap(this.sourceBitmap1, this.targetImagePictureBox.Width);
                }
                else if(sender == this.loadSourceImage2Button)
                {
                    StreamReader streamReader = new StreamReader(openFileDialog.FileName);

                    this.sourceBitmap2 = Bitmap.FromStream(streamReader.BaseStream) as Bitmap;

                    streamReader.Close();

                    this.sourceImage2PictureBox.Image = BitmapHelper.CopyBitmap(this.sourceBitmap2, this.targetImagePictureBox.Width);
                }

                ApplyFilter(true);
            }
        }

        #endregion
        #region 블렌딩 타입 콤보 박스 선택 인덱스 변경시 처리하기 - blendTypeComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        /// 블렌딩 타입 콤보 박스 선택 인덱스 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void blendTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ApplyFilter(true);
        }

        #endregion
        #region 이미지 저장 버튼 클릭시 처리하기 - saveImageButton_Click(sender, e)

        /// <summary>
        /// 이미지 저장 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void saveImageButton_Click(object sender, EventArgs e)
        {
            ApplyFilter(false);

            if(this.targetBitmap != null)
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog();

                saveFileDialog.Title  = "이미지 파일 저장";
                saveFileDialog.Filter = "PNG 파일(*.png)|*.png|JPEG 파일(*.jpg)|*.jpg|비트맵 파일(*.bmp)|*.bmp";

                if(saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    string fileExtension = Path.GetExtension(saveFileDialog.FileName).ToUpper();

                    ImageFormat imageFormat = ImageFormat.Png;

                    if(fileExtension == "BMP")
                    {
                        imageFormat = ImageFormat.Bmp;
                    }
                    else if(fileExtension == "JPG")
                    {
                        imageFormat = ImageFormat.Jpeg;
                    }

                    StreamWriter streamWriter = new StreamWriter(saveFileDialog.FileName, false);

                    targetBitmap.Save(streamWriter.BaseStream, imageFormat);

                    streamWriter.Flush();
                    streamWriter.Close();

                    targetBitmap = null;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 필터 적용하기 - ApplyFilter(preview)

        /// <summary>
        /// 필터 적용하기
        /// </summary>
        /// <param name="preview">미리보기 여부</param>
        private void ApplyFilter(bool preview)
        {
            if(this.sourceBitmap1 == null || this.sourceBitmap2 == null || this.sourceImage2PictureBox.Image == null || this.sourceImage1PictureBox.Image == null)
            {
                return;
            }

            if(preview == true)
            {
                this.targetImagePictureBox.Image = BitmapHelper.BelndBitmap
                (
                    this.sourceImage1PictureBox.Image as Bitmap,
                    this.sourceImage2PictureBox.Image as Bitmap,
                    (BitwiseBlendType)this.blueBlendTypeComboBox.SelectedItem,
                    (BitwiseBlendType)this.greenBlendTypeComboBox.SelectedItem,
                    (BitwiseBlendType)this.redBlendTypeComboBox.SelectedItem
                );
            }
            else
            {
                this.targetBitmap = BitmapHelper.BelndBitmap
                (
                    this.sourceBitmap1,
                    this.sourceBitmap2,
                    (BitwiseBlendType)blueBlendTypeComboBox.SelectedItem,
                    (BitwiseBlendType)greenBlendTypeComboBox.SelectedItem,
                    (BitwiseBlendType)redBlendTypeComboBox.SelectedItem
                );
            }
        }

        #endregion
    }
}